/*
 * Copyright (c) 2016, Adam <Adam@anope.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.rizon.plexus.commands;

import java.util.Random;
import net.rizon.plexus.Client;
import net.rizon.plexus.Message;
import net.rizon.plexus.Numeric;
import net.rizon.plexus.Parameter;
import net.rizon.plexus.command.Command;
import net.rizon.plexus.ident.IdentClient;
import net.rizon.plexus.messages.PingMessage;
import net.rizon.plexus.users.LocalUser;

public class User extends Command
{
	@Parameter(index = 0, required = true)
	private String username;

	@Parameter(index = 1, required = true)
	private String clientHost;

	@Parameter(index = 2, required = true)
	private String serverHost;

	@Parameter(index = 3, required = true)
	private String gecos;
	
	public User()
	{
		super("USER");
	}
	
	@Override
	public void executeUnregistered(Client client)
	{
		int i = username.indexOf('@');
		if (i != -1)
			username = username.substring(0, i);
		
		if (getPlexus().getConf().getGeneral().isPing_cookie() && client.getPingCookie() == null)
		{
			Random random = new Random();
			int pingCookie = random.nextInt();

			client.setPingCookie(pingCookie);

			Message message = new PingMessage(null, String.format("%x", pingCookie).toUpperCase());
			client.writeMessage(message);
		}
		
		if (client.getUsername() == null || client.getUsername().startsWith(IdentClient.NO_IDENT_PREFIX))
			client.setUsername(IdentClient.NO_IDENT_PREFIX + username);

		client.setClientHost(clientHost);
		client.setServerHost(serverHost);
		client.setGecos(gecos);
		
		client.register();
	}
	
	@Override
	public void execute(LocalUser user)
	{
		user.writeNumeric(Numeric.ERR_ALREADYREGISTRED);
	}
}