/*
 * Copyright (c) 2016, Adam <Adam@anope.org>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.rizon.plexus.numerics;

import net.rizon.plexus.BuildableNumeric;
import net.rizon.plexus.Entity;
import net.rizon.plexus.Message;
import net.rizon.plexus.Numeric;
import net.rizon.plexus.Parameter;

public class RplWhoisChannels extends Message implements BuildableNumeric
{
	@Parameter(index = 0)
	private final String target;

	@Parameter(index = 1)
	private String channels;

	public RplWhoisChannels(Entity source, String target)
	{
		super(source, Numeric.RPL_WHOISCHANNELS);

		this.target = target;
	}

	@Override
	public String toString()
	{
		return "RplWhoisChannels{" + "target=" + target + ", channels=" + channels + '}';
	}

	public String getTarget()
	{
		return target;
	}

	public String getChannels()
	{
		return channels;
	}

	@Override
	public String getReplacement()
	{
		return channels;
	}

	@Override
	public void setReplacement(String str)
	{
		channels = str;
	}

	@Override
	public int getLength()
	{
		return target.length();
	}

	@Override
	public int getArgs()
	{
		return 1;
	}

	@Override
	public boolean wrapMaxArgs()
	{
		return false;
	}
}
